*-----------------------------------------------------------
* Title      : CRC32, 32bit words for PCMHammer
* Written by : Antus @ PCMHacking.net
* Date       : 2023-03-24
* Description: Based on crcSlow() configued for 32bit
*              from https://barrgroup.com/downloads/code-crc-c
*              CRC32 of data in 000000 - 003FFF in D0 at completion.
*              Expected CRC is AB5A2C04 for 0-3FFF
*
* 060000-07FFFF	7021E66E OperatingSystem
* 040000-05FFFF	CF812780 OperatingSystem
* 020000-03FFFF	D4A2F10A OperatingSystem
* 008000-01FFFF	74EEAE5C Calibration
* 006000-007FFF	85B5BB36 Parameter
* 004000-005FFF	9907B305 Parameter
* 000000-003FFF	AB5A2C04 Boot
*-----------------------------------------------------------

START   ORG     $FF8000                 ; load address of program
                move.l  #$000000,a0     ; start addres is $000000
                move.l  #$004000,a1     ; process $4000 bytes
                lea     (a0,a1.l),a1    ; for (int byte = 0; byte < nBytes; ++byte)
                clr.l   d0              ; d0 = byte
                moveq   #$18,d2         ; WIDTH-8 to d2

mainloop:
                clr.l   d1              ; unsigned int remainder=0
                move.b  (a0)+,d1        ; message[byte] to register d1
                lsl.l   d2,d1           ; shift left copy of message[byte] by (WIDTH-8)
                eor.l   d1,d0           ; xor in to remainder
                moveq   #8,d1           ; for (unsigned char bit = 8; bit > 0; --bit)

bitloop:
                tst.l   d0              ; if (remainder & TOPBIT)
                add.l   d0,d0           ; remainder = (remainder << 1); (optimised by tricky use of an add opcode)
                blt.s   polynomial      ; then jump to the polynomial part              
                subq.b  #1,d1           ; next bit
                bne.s   bitloop         ; any bits left?

mainloop_check:
                cmpa.l  a0,a1           ; is byte < nBytes?
                bne.s   mainloop        ; if not, iterate main loop
                
stop:           stop #$2700             ; stop execution (use rts in real pcm)

polynomial:
                eori.l  #$4C11DB7,d0 ; ^ POLYNOMIAL;
                subq.b  #1,d1           ; next byte
                bne.s   bitloop         ; is there another bit in this byte? (loop)
                bra.s   mainloop_check  ; next main loop

        END START





*~Font name~Courier New~
*~Font size~10~
*~Tab type~1~
*~Tab size~4~
